from pathlib import Path

from great_expectations.core.id_dict import BatchSpec
from great_expectations.execution_engine import PandasExecutionEngine
from odd_great_expectations.logger import logger
from oddrn_generator.generators import FilesystemGenerator


def get_file_dataset(engine: PandasExecutionEngine) -> list[str]:
    batch_spec: BatchSpec = engine.batch_manager.active_batch.batch_spec

    logger.debug("Batch spec")
    logger.debug(batch_spec.__dict__)
    logger.debug(f"Resolved path: {Path(batch_spec['path']).resolve()}")

    batch_path = str(Path(batch_spec["path"]).resolve())
    generator = FilesystemGenerator(host_settings="local", path=batch_path)

    path = generator.get_oddrn_by_path("path")
    logger.debug(f"Resulted path: {path}")

    return [path]



import os
import sys

import loguru


def _create_logger(level: str = "INFO"):
    log_level = os.getenv("LOGLEVEL", level)
    root_logger = loguru.logger

    if log_level is not None:
        try:
            root_logger.remove(0)
            root_logger.add(sys.stderr, level=log_level)
        except Exception as e:
            root_logger.add(sys.stderr, level="DEBUG")
            return root_logger

    return root_logger


logger = _create_logger()



from typing import List

from great_expectations.execution_engine.sqlalchemy_execution_engine import (
    SqlAlchemyBatchData,
    SqlAlchemyExecutionEngine,
)
from loguru import logger
from oddrn_generator import MssqlGenerator, PostgresqlGenerator, SnowflakeGenerator
from sqlalchemy.engine import Engine


def postgres_dataset(engine: Engine, batch_data: SqlAlchemyBatchData) -> str:
    generator_params = {
        "host_settings": engine.url.host,
        "databases": engine.url.database,
        "schemas": batch_data.source_schema_name or "public",
        "tables": batch_data.source_table_name,
    }

    generator = PostgresqlGenerator(**generator_params)
    return generator.get_oddrn_by_path("tables")


def snowflake_dataset(engine: Engine, batch_data: SqlAlchemyBatchData) -> str:
    # Snowflake might create SqlAlchemy database name like <DATABASE_NAME>/<SCHEMA_NAME>
    generator_params = {
        "host_settings": engine.engine.url.host,
        "databases": engine.engine.url.database.split("/")[0],
        "schemas": batch_data.source_schema_name or "public",
        "tables": batch_data.source_table_name,
    }

    generator = SnowflakeGenerator(**generator_params)
    oddrn = generator.get_oddrn_by_path("tables")
    logger.info(f"Snowflake source {oddrn=}")
    return oddrn


def mssql_dataset(engine: Engine, batch_data: SqlAlchemyBatchData) -> str:
    generator_params = {
        "host_settings": engine.url.host,
        "databases": engine.url.database,
        "schemas": batch_data.source_schema_name or "dbo",
        "tables": batch_data.source_table_name,
    }

    generator = MssqlGenerator(**generator_params)
    oddrn = generator.get_oddrn_by_path("tables")
    logger.info(f"MSSQL source {oddrn=}")
    return oddrn


def get_sql_table_dataset(exec_engine: SqlAlchemyExecutionEngine) -> List[str]:
    dialect_name = exec_engine.dialect_name
    engine = exec_engine.engine
    batch_data = exec_engine.batch_manager.active_batch_data

    if dialect_name == "postgresql":
        return [postgres_dataset(engine, batch_data)]
    elif dialect_name == "snowflake":
        return [snowflake_dataset(engine, batch_data)]
    elif dialect_name == "mssql":
        return [mssql_dataset(engine, batch_data)]

    raise ValueError(f"Unknown {dialect_name=}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['odd_great_expectations', 'odd_great_expectations.dataset']

package_data = \
{'': ['*']}

install_requires = \
['funcy>=2.0,<3.0',
 'great-expectations>=0.18.12,<0.19.0',
 'loguru>=0.7.2,<0.8.0',
 'odd-models==2.0.48',
 'oddrn-generator>=0.1.68,<0.2.0',
 'psycopg2-binary>=2.9.5,<3.0.0',
 'sqlalchemy>=1.4.46,<2.0.0']

setup_kwargs = {
    'name': 'odd-great-expectations',
    'version': '0.2.2',
    'description': 'OpenDataDiscovery Action for Great Expectations',
    'long_description': "## OpenDataDiscovery Action for handling Great Expectations tests results.\n\n[![PyPI version](https://badge.fury.io/py/odd-great-expectations.svg)](https://badge.fury.io/py/odd-great-expectations)\n\n![image](assets/screenshot.png)\n\n# What is it?\n`odd_great_expectation.action.ODDAction`\nIs a class derived from GX `ValidationAction` which will be run by GreatExpectations at runtime with `ValidationResult`s for checkpoint.\n\n\n# How to use it?:\nInstall `odd-great-expectations` package\n```bash\npip install odd-great-expectations\n```\nAdd `ODDAction` action to some checkpoint's action list:\n```yaml\nname: <CHECKPOINT_NAME>\nconfig_version: 1.0\n...\naction_list:\n  # other actions\n  - name: store_metadata_to_odd\n    action:\n      module_name: odd_great_expectations.action\n      class_name: ODDAction\n      platform_host: <PLATFORM_HOST>\n      data_source_name: <DATA_SOURCE_NAME>\n```\n\nParameters:\n\n`platform_host` - Location of OpenDataDiscovery platform, i.e. http://localhost:8080\n\n`platform_token` - OpenDataDiscovery token, how to get it - https://docs.opendatadiscovery.org/configuration-and-deployment/trylocally#create-collector-entity\n\n`data_source_name` - Unique name for data source, i.e. local_qa_test\n\nBoth `platform_host` and `platform_token`  can be set using `ODD_PLATFORM_HOST` and `ODD_PLATFORM_PLATFORM` env variables accordingly.\n\nRun checkpoint\n```bash\ngreat_expectations checkpoint run <CHECKPOINT_NAME>\n```\nCheck results on `PLATFORM_HOST` UI.\n\n## Supporting features\n| Feature                     | Supporting |\n| --------------------------- | ---------- |\n| V3 API +                    | +          |\n| SqlAlchemyEngine            | +          |\n| PandasEngine                | +          |\n| Great Expectations V2 API - | -          |\n| Cloud Solution              | -          |\n",
    'author': 'Pavel Makarichev',
    'author_email': 'vixtir90@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
